//
//  CrashReportUploadOperation.m
//  HelloAnalytics
//
//  Created by Saravanan S on 08/08/18.
//  Copyright © 2018 Saravanan S. All rights reserved.
//

#import "APCrashReportUploadOperation.h"

#if __has_include(<Apptics/Apptics.h>)
#import <Apptics/Apptics.h>
//#import <Apptics/APLog.h>
#import <Apptics/ZASystemInfo.h>
#import <Apptics/ZAFileManager.h>
#import <Apptics/ZANetworking.h>
#import <Apptics/ZAPIIManager.h>
#import <Apptics/APJWTWrapper.h>
#import <Apptics/NSData+APGunZip.h>
#import <Apptics/APAAAUtil.h>
#endif

@interface APCrashReportUploadOperation()
@property (nonatomic, strong) APAAAUtil *aaaUtil;
@end

@implementation APCrashReportUploadOperation

NSString *notificationName=@"APFLTCrashUploadSuccessNotification";

- (id)initWithCrashReport:(NSDictionary *)crashReport{
  self = [super init];
  
  if (!self) return nil;
  
  NSMutableDictionary *crash = (NSMutableDictionary*)[crashReport mutableCopy];
    [self setReportIds:(crashReport[@"reportIDs"]) ? crashReport[@"reportIDs"] : @[]];
    [crash removeObjectForKey:@"reportIDs"];
      
    NSString *userId = crashReport[@"userid"];
    NSString *anonid = crashReport[@"anonid"];
    NSNumber *isfatal = crashReport[@"isfatal"];
    
    [crash removeObjectForKey:@"userid"];
    [crash removeObjectForKey:@"anonid"];
    [crash removeObjectForKey:@"isfatal"];
    [crash removeObjectForKey:@"size"];
    
    NSString *params = [[ZANetworking sharedManager] setDefaultParams];
    
    params = [[ZANetworking sharedManager] setIdentificationParams:params userID:userId anonID:anonid];
    if (isfatal.boolValue == true){
        notificationName=@"APFLTCrashUploadSuccessNotification";
    }else{
        notificationName=@"APFLTNonFatalsUploadSuccessNotification";
    }
    self.aaaUtil = [[ZANetworking sharedManager] getAuthindicatorForUserID:userId anonID:anonid];
//    if (za_isNotEmpty(anonid)){
//        params = [NSString stringWithFormat:@"anondeviceid=%@", anonid];
//    }else{
//        params = [NSString stringWithFormat:@"deviceid=%@&userid=%@", [ZAPIIManager getInstance].regDeviceId, userId];
//    }
        
  self.params = params;
  [self setCrashReport:crash];
  return self;
}

- (id)initWithFileName:(NSString *)fileName{
  self = [super init];
  
  if (!self) return nil;
  [self setFileName:fileName];
  return self;
}

- (void)dealloc {
  _fileName = nil;
}

- (void)main {
  [self sendReports:self.crashReport];
}

-(void) parseData : (NSDictionary *) response withSuccess : (void (^)(long long value))success;{
    NSString *result = response[@"result"];
    if ([result isEqualToString:@"success"]){
      success(1);
      return;
    }else{
    [ZANetworking processError:response aaaUtil:self.aaaUtil];
      success(0);
      return;
    }
}

-(void) sendReports:(NSDictionary *)report
{
  // TODO: Write a priority based network layer here
  // Interwebzz code goes here
    [self.aaaUtil getTokenWithSuccess:^(NSString * _Nonnull token) {
  NSURLSessionConfiguration *sessionConfig = [NSURLSessionConfiguration defaultSessionConfiguration];
  sessionConfig.timeoutIntervalForRequest = 120.0;//2min allowance; iOS default is 60 seconds.
  sessionConfig.timeoutIntervalForResource = 120.0;//2min; iOS Default is 7 days
  
  NSURLSession *session = [NSURLSession sessionWithConfiguration:sessionConfig];
        NSDictionary *crashDictionary = report;
    NSString *urlString = self.url.absoluteString;
    urlString = [NSString stringWithFormat:@"%@?%@", urlString, self.params];
  NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:urlString]];
  
  if ([ZANetworking sharedManager].userAgent && ![[ZANetworking sharedManager].userAgent isEqualToString:@""])
  {
    [request setValue:[ZANetworking sharedManager].userAgent forHTTPHeaderField:@"User-Agent"];
  }
  
  [[ZANetworking sharedManager] setHTTPHeaderField:request forPath:JURLPathCrash];
  [request setValue:[NSString stringWithFormat:@"Bearer %@",token] forHTTPHeaderField:@"Authorization"];
  
    if ([ZAFileManager sharedInstance].enableGzip){
        [request setValue:@"gzip" forHTTPHeaderField:@"Content-Encoding"];
    }else{
        [request addValue:@"text/plain" forHTTPHeaderField:@"Content-Type"];
    }
        
  request.HTTPMethod = @"POST";
  request.HTTPShouldHandleCookies = false;
  NSError *error;
#if DEBUG
  NSLog(@"Http headers -> %@", request.allHTTPHeaderFields);
#endif
  @try {
    //NSData *POSTHttpBody = [[APJWTWrapper sharedManager] encodeWithJWT:crashDictionary];
      NSData *postData = [NSJSONSerialization dataWithJSONObject:crashDictionary options:0 error:nil];
      NSData *POSTHttpBody;
      if ([ZAFileManager sharedInstance].enableGzip){
          NSData *compressedData = [postData ap_gzippedDataWithCompressionLevel:1.0];
#if DEBUG
              if (!compressedData) {
                  NSLog(@"Compression failed");
              }
#endif
          POSTHttpBody = [[APJWTWrapper sharedManager] encodeDataWithJWT:compressedData];
      }else{
          POSTHttpBody = [[APJWTWrapper sharedManager] encodeWithJWT:crashDictionary];
      }
    NSString *postLength = [NSString stringWithFormat:@"%lu", (unsigned long)[POSTHttpBody length]];
    [request setValue:postLength forHTTPHeaderField:@"Content-Length"];
    [request setHTTPBody:POSTHttpBody];
  }
  @catch (NSException *exception) {
#if DEBUG
    NSLog(@"%@ exception encoding crash data: %@", self, exception);
#endif
  }
  if (error){
#if DEBUG
    NSLog(@"%@ error encoding crash data: %@", self, error);
#endif
  }else{
#if DEBUG
      NSLog(@"Request made -> %@",request.URL.absoluteString);
      NSLog(@"Http headers -> %@", request.allHTTPHeaderFields);
#endif
      [self beginBackgroundUpdateTask];
      [[session dataTaskWithRequest:request completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error)
        {
#if DEBUG
          NSLog(@"Response received -> %@",response.URL.absoluteString);
          NSLog(@"Response data -> %@",[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding]);
#endif
          if (!error)
          {
            NSError *jsonParsingError;
            NSDictionary *responseJSON = [NSJSONSerialization JSONObjectWithData:data options:0 error:&jsonParsingError];
            
            if (responseJSON)
            {
#if DEBUG
              NSLog(@"Data received -> %@",responseJSON);
#endif
              [self parseData:responseJSON withSuccess:^(long long value) {
                if (value > 0){
//                  #if KSCR_PRIVATE || PLCR_PRIVATE
                  [[NSNotificationCenter defaultCenter] postNotificationName:notificationName object:self.reportIds];
//                  #else
//                  [[NSNotificationCenter defaultCenter] postNotificationName:@"APFLTCrashUploadSuccessNotification" object:self.fileName];
//                  #endif
                }
              }];
            }else{
#if DEBUG
              NSLog(@"Error while parsing crash upload response -> %@",jsonParsingError);
#endif
            }
          }
          else
          {
#if DEBUG
            NSLog(@"Error while uploading crash report -> %@",error);
#endif
          }
          [self endBackgroundUpdateTask];
        }]resume];
  }
    } andFailure:^(NSError * _Nonnull error) {
#if DEBUG
        NSLog(@"Error while uploading crash report -> %@",error);
#endif
    }];
}

- (void) beginBackgroundUpdateTask
{
#if !TARGET_OS_OSX && !TARGET_OS_WATCH
    self.backgroundUpdateTask = [[UIApplication performSelector:@selector(sharedApplication)] beginBackgroundTaskWithExpirationHandler:^{
        [self endBackgroundUpdateTask];
    }];
#endif
}

- (void) endBackgroundUpdateTask
{
#if !TARGET_OS_OSX && !TARGET_OS_WATCH
    [[UIApplication performSelector:@selector(sharedApplication)] endBackgroundTask: self.backgroundUpdateTask];
    self.backgroundUpdateTask = UIBackgroundTaskInvalid;
#endif
}
@end
