#import "AppticsFlutterPlugin.h"
#import "APCrashReportUploadOperation.h"

#define maxDataSize 1.8
#define rt_safeDictionary(A)  ({ NSDictionary *__a = A; (__a != nil) ? __a : @{}; })

#if __has_include(<AppticsFeedbackKit/AppticsFeedbackKit.h>)
#import <AppticsFeedbackKit/AppticsFeedbackKit.h>
#define __HAS_APPTICS_FEEDBACKKIT_FRAMEWORK__
#endif

@implementation AppticsFlutterPlugin

UIWindow *window;
NSUInteger childviewcontrollers;
NSNumber *option;
NSString *storeurl;
NSNumber *reminder;
NSString *updateKey;
NSNumber *updateId;
NSDictionary *customproperties;

bool showStorePromptOnFulFillingCriteria;
bool disableIfNotInstalledFromAppStore;
NSNumber *maxTimesToShowPopup;
FlutterResult _result;

- (void)cancelledReviewPrompt{
    if (_result){
        _result(@0);
        _result = nil;
    }
}

- (void)willDisplayReviewPrompt{
    if (_result){
        _result([APStoreReviewManager getInstance].criteriaId);
        _result = nil;
    }else{
        [[APStoreReviewManager getInstance] showAppStoreReviewPrompt];
    }
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    
    AppticsConfig.defaultConfig.enableRateUs=true;
    [Apptics initializeWithVerbose:true];
    [APRateUs enable:true];
    [APRemoteConfig enable: true];

    FlutterMethodChannel* channel = [FlutterMethodChannel
                                     methodChannelWithName:@"apptics_flutter"
                                     binaryMessenger:[registrar messenger]];
    AppticsFlutterPlugin* instance = [[AppticsFlutterPlugin alloc] init];
    [AppticsCrashFlutterPlugin shared];
    [Apptics setUserConsentPresentCompletionHandler:^(bool status, NSError *error) {
        
    }];
    
    [Apptics setUserConsentDismissCompletionHandler:^(int status) {
        [instance openSettings];
    }];
    [registrar addMethodCallDelegate:instance channel:channel];

    [ARCustomHandlerManager setCustomHandler:instance];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([@"enableAutomaticEventTracking" isEqualToString:call.method]) {
        [self enableAutomaticEventTracking:call result:result];
    } else if ([@"enableAutomaticScreenTracking" isEqualToString:call.method]) {
        [self enableAutomaticScreenTracking:call result:result];
    } else if ([@"enableAutomaticCrashTracking" isEqualToString:call.method]) {
        [self enableAutomaticCrashTracking:call result:result];
    } else if ([@"setCrashCustomProperty" isEqualToString:call.method]) {
        [self setCustomProperties:call result:result];
    } else if ([@"enableReviewAndSendCrashReport" isEqualToString:call.method]) {
        [self enableReviewAndSendCrashReport:call result:result];
    } else if ([@"getPrivacyStatus" isEqualToString:call.method]) {
        [self getPrivacyStatus:call result:result];
    } else if ([@"presentPrivacyReviewPopup" isEqualToString:call.method]) {
        [self presentPrivacyReviewPopup:call result:result];
    } else if ([@"resetConsentPreference" isEqualToString:call.method]) {
        [self resetConsentPreference:call result:result];
    } else if ([@"reset" isEqualToString:call.method]) {
        [self reset:call result:result];
    } else if ([@"flush" isEqualToString:call.method]) {
        [self flush:call result:result];
    } else if ([@"trackSignUp" isEqualToString:call.method]) {
        [self trackSignUp:call result:result];
    } else if ([@"trackLogIn" isEqualToString:call.method]) {
        [self trackLogIn:call result:result];
    } else if ([@"trackLogOut" isEqualToString:call.method]) {
        [self trackLogOut:call result:result];
    } else if ([@"trackSignUpWithGroupID" isEqualToString:call.method]) {
        [self trackSignUpWithGroupID:call result:result];
    } else if ([@"trackLogInWithGroupID" isEqualToString:call.method]) {
        [self trackLogInWithGroupID:call result:result];
    } else if ([@"trackLogOutWithGroupID" isEqualToString:call.method]) {
        [self trackLogOutWithGroupID:call result:result];
    } else if ([@"setUser" isEqualToString:call.method]) {
        [self setCurrentUser:call result:result];
    } else if ([@"setCurrentUserWithGroupId" isEqualToString:call.method]) {
        [self setCurrentUserWithGroupId:call result:result];
    } else if ([@"openPrivacySettings" isEqualToString:call.method]) {
        [self openPrivacySettings:call result:result];
    } else if ([@"getTrackingState" isEqualToString:call.method]) {
        [self getTrackingState:call result:result];
    } else if ([@"setTrackingState" isEqualToString:call.method]) {
        [self setTrackingState:call result:result];
    } else if ([@"trackingStatus" isEqualToString:call.method]) {
        [self trackingStatus:call result:result];
    } else if ([@"setTrackingStatus" isEqualToString:call.method]) {
        [self setTrackingStatus:call result:result];
    } else if ([@"crashReportStatus" isEqualToString:call.method]) {
        [self crashReportStatus:call result:result];
    } else if ([@"setCrashReportStatus" isEqualToString:call.method]) {
        [self setCrashReportStatus:call result:result];
    } else if ([@"personalInfoTrackingStatus" isEqualToString:call.method]) {
        [self personalInfoTrackingStatus:call result:result];
    } else if ([@"setPersonalInfoTrackingStatus" isEqualToString:call.method]) {
        [self setPersonalInfoTrackingStatus:call result:result];
    } else if ([@"isUserLoggedIn" isEqualToString:call.method]) {
        [self isUserLoggedIn:call result:result];
    } else if ([@"setSendAnalyticsDataUsingMobileData" isEqualToString:call.method]) {
        [self setSendAnalyticsDataUsingMobileData:call result:result];
    } else if ([@"sendAnalyticsDataUsingMobileData" isEqualToString:call.method]) {
        [self sendAnalyticsDataUsingMobileData:call result:result];
    } else if ([@"getLocalizableStringForKey" isEqualToString:call.method]) {
        [self getLocalizableStringForKey:call result:result];
    } else if ([@"setDefaultLanguage" isEqualToString:call.method]) {
        [self setDefaultLanguage:call result:result];
    } else if ([@"setCurrentLocation" isEqualToString:call.method]) {
        [self setCurrentLocation:call result:result];
    } else if ([@"setPrivacyConsentType" isEqualToString:call.method]) {
        [self setPrivacyConsentType:call result:result];
    } else if ([@"setTypeOfAnonymity" isEqualToString:call.method]) {
        [self setTypeOfAnonymity:call result:result];
    } else if ([@"enableAutoCheckForAppUpdate" isEqualToString:call.method]) {
        [self enableAutoCheckForAppUpdate:call result:result];
    } else if ([@"crash" isEqualToString:call.method]) {
        [self crash:call result:result];
    } else if ([@"isReachable" isEqualToString:call.method]) {
        [self isReachable:call result:result];
    } else if ([@"addEvent" isEqualToString:call.method]) {
        [self addEvent:call result:result];
    } else if ([@"screenAttached" isEqualToString:call.method]) {
        [self trackViewEnter:call result:result];
    } else if ([@"screenDetached" isEqualToString:call.method]) {
        [self trackViewExit:call result:result];
    } else if ([@"checkAndUpdateAlert" isEqualToString:call.method]) {
        [self checkAndUpdateAlert:call result:result];
    } else if ([@"onSendImpressionStatus" isEqualToString:call.method]) {
        [self onSendImpressionStatus:call result:result];
    } else if ([@"onClickUpdate" isEqualToString:call.method]) {
        [self onClickUpdate:call result:result];
    } else if ([@"onClickReminder" isEqualToString:call.method]) {
        [self onClickReminder:call result:result];
    } else if ([@"onClickIgnore" isEqualToString:call.method]) {
        [self onClickIgnore:call result:result];
    } else if ([@"onClickNonSupportAlert" isEqualToString:call.method]) {
        [self onClickNonSupportAlert:call result:result];
    } else if ([@"checkForRatingPop" isEqualToString:call.method]) {
        [self checkForRatingPop:call result:result];
    } else if ([@"isAppticsFeedbackModuleAvailable" isEqualToString:call.method]) {
        [self isAppticsFeedbackModuleAvailable:call result:result];
    } else if ([@"openFeedbackScreen" isEqualToString:call.method]) {
        [self openFeedback:call result:result];
    } else if ([@"openPlayStore" isEqualToString:call.method]) {
        [self openPlayStore:call result:result];
    } else if ([@"daysBeforeShowingPopupAgain" isEqualToString:call.method]) {
        [self daysBeforeShowingPopupAgain:call result:result];
    } else if ([@"maxTimesToShowPopup" isEqualToString:call.method]) {
        [self maxTimesToShowPopup:call result:result];
    } else if ([@"showStoreAlertOnFulFillingCriteria" isEqualToString:call.method]) {
        [self showStoreAlertOnFulFillingCriteria:call result:result];
    } else if ([@"disableIfNotInstalledFromPlayStore" isEqualToString:call.method]) {
        [self disableIfNotInstalledFromPlayStore:call result:result];
    } else if ([@"setDaysBeforeShowingPopupAgain" isEqualToString:call.method]) {
        [self setDaysBeforeShowingPopupAgain:call result:result];
    } else if ([@"setMaxTimesToShowPopup" isEqualToString:call.method]) {
        [self setMaxTimesToShowPopup:call result:result];
    } else if ([@"setShowStoreAlertOnFulFillingCriteria" isEqualToString:call.method]) {
        [self setShowStoreAlertOnFulFillingCriteria:call result:result];
    } else if ([@"setDisableIfNotInstalledFromPlayStore" isEqualToString:call.method]) {
        [self setDisableIfNotInstalledFromPlayStore:call result:result];
    } else if ([@"sentRatingStats" isEqualToString:call.method]) {
        [self sentStats:call result:result];
    } else if ([@"updateRatingShown" isEqualToString:call.method]) {
        [self updateRatingShown:call result:result];
    } else if ([@"getStringValue" isEqualToString:call.method]) {
        [self getStringValue:call result:result];
    } else if ([@"setCustomConditionValue" isEqualToString:call.method]) {
        [self setCustomConditionValue:call result:result];
    }
    else if ([@"crashTracker" isEqualToString:call.method]) {
        [self crashTracker:call result:result];
    }
    else
    {
        result(FlutterMethodNotImplemented);
    }
}

- (void)enableAutomaticEventTracking:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *status = call.arguments[@"status"];
    [Apptics enableAutomaticEventTracking:status.boolValue];
    result(nil);
}

- (void)enableAutomaticScreenTracking:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *status = call.arguments[@"status"];
    [Apptics enableAutomaticScreenTracking:status.boolValue];
    result(nil);
}

- (void)enableAutomaticCrashTracking:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *status = call.arguments[@"status"];
    [Apptics enableAutomaticCrashTracking:status.boolValue];
    result(nil);
}

- (void)setCustomProperties:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSDictionary *custom_properties = call.arguments[@"properties"];
    customproperties = custom_properties;
    [Apptics setCrashCustomProperty:custom_properties];
    result(nil);
}

- (void)enableReviewAndSendCrashReport:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *status = call.arguments[@"status"];
    [Apptics enableReviewAndSendCrashReport:status.boolValue];
    result(nil);
}

- (void)getPrivacyStatus:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *status = @([Apptics getPrivacyStatus]);
    result(status);
}

- (void)presentPrivacyReviewPopup:(FlutterMethodCall*)call result:(FlutterResult)result{
    [Apptics showPrivacyConsent];
    result(nil);
}

- (void)resetConsentPreference:(FlutterMethodCall*)call result:(FlutterResult)result{
    [Apptics resetConsentPreference];
    result(nil);
}

- (void)reset:(FlutterMethodCall*)call result:(FlutterResult)result{
    [Apptics reset];
    result(nil);
}

- (void)flush:(FlutterMethodCall*)call result:(FlutterResult)result{
    [Apptics flush];
    result(nil);
}

- (void)trackSignUp:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *userID = call.arguments[@"user_id"];
    [Apptics trackSignUp:userID];
    result(nil);
}

- (void)trackLogIn:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *userID = call.arguments[@"user_id"];
    [Apptics trackLogIn:userID];
    result(nil);
}

- (void)trackLogOut:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *userID = call.arguments[@"user_id"];
    [Apptics trackLogOut:userID];
    result(nil);
}

- (void)trackSignUpWithGroupID:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *userID = call.arguments[@"user_id"];
    NSString *groupID = call.arguments[@"group_id"];
    [Apptics trackSignUp:userID groupId:groupID];
    result(nil);
}

- (void)trackLogInWithGroupID:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *userID = call.arguments[@"user_id"];
    NSString *groupID = call.arguments[@"group_id"];
    [Apptics trackLogIn:userID groupId:groupID];
    result(nil);
}

- (void)trackLogOutWithGroupID:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *userID = call.arguments[@"user_id"];
    NSString *groupID = call.arguments[@"group_id"];
    [Apptics trackLogOut:userID groupId:groupID];
    result(nil);
}

- (void)setCurrentUser:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *userID = call.arguments[@"user_id"];
    [Apptics setCurrentUser:userID];
    result(nil);
}

- (void)setCurrentUserWithGroupId:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *userID = call.arguments[@"user_id"];
    NSString *groupID = call.arguments[@"group_id"];
    [Apptics setCurrentUser:userID groupId:groupID];
    result(nil);
}

-(void) openSettings{
    window = [[UIWindow alloc] initWithFrame:[UIScreen mainScreen].bounds];
    UIViewController *rootViewController = [UIViewController new];
    UINavigationController *navigationController = [[UINavigationController alloc] initWithRootViewController:rootViewController];
    navigationController.delegate = self;
    
    window.rootViewController = navigationController;
    [window makeKeyAndVisible];
    //        window.backgroundColor = [UIColor blueColor];
    double delayInSeconds = 0.2;
    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
        [Apptics openAnalyticSettingsController: navigationController];
    });
}

- (void)openPrivacySettings:(FlutterMethodCall*)call result:(FlutterResult)result{
    [self openSettings];
    result(nil);
}

- (NSNumber*) getTrackingState{
    if ([Apptics trackingStatus] == true && [Apptics crashReportStatus] == true && [Apptics personalInfoTrackingStatus] == true){
        return @(1);
    }else if ([Apptics trackingStatus] == true && [Apptics crashReportStatus] == true && [Apptics personalInfoTrackingStatus] == false){
        return @(4);
    }else if ([Apptics trackingStatus] == true && [Apptics personalInfoTrackingStatus] == true){
        return @(2);
    }else if ([Apptics trackingStatus] == true && [Apptics personalInfoTrackingStatus] == false){
        return @(5);
    }else if ([Apptics crashReportStatus] == true && [Apptics personalInfoTrackingStatus] == true){
        return @(3);
    }else if ([Apptics crashReportStatus] == true && [Apptics personalInfoTrackingStatus] == false){
        return @(6);
    }else{
        return @0;
    }
}

- (void)getTrackingState:(FlutterMethodCall*)call result:(FlutterResult)result{
    result([self getTrackingState]);
}

- (void)setTrackingState:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *status = call.arguments[@"status"];
    if ([status isEqual: @(1)]){
        [Apptics setTrackingStatus:true];
        [Apptics setCrashReportStatus:true];
        [Apptics setPersonalInfoTrackingStatus:true];
    }else if ([status isEqual: @(2)]){
        [Apptics setTrackingStatus:true];
        [Apptics setCrashReportStatus:false];
        [Apptics setPersonalInfoTrackingStatus:true];
    }else if ([status isEqual: @(3)]){
        [Apptics setTrackingStatus:false];
        [Apptics setCrashReportStatus:true];
        [Apptics setPersonalInfoTrackingStatus:true];
        
    }else if ([status isEqual: @(4)]){
        [Apptics setTrackingStatus:true];
        [Apptics setCrashReportStatus:true];
        [Apptics setPersonalInfoTrackingStatus:false];
    }else if ([status isEqual: @(5)]){
        [Apptics setTrackingStatus:true];
        [Apptics setCrashReportStatus:false];
        [Apptics setPersonalInfoTrackingStatus:false];
    }else if ([status isEqual: @(6)]){
        [Apptics setTrackingStatus:false];
        [Apptics setCrashReportStatus:true];
        [Apptics setPersonalInfoTrackingStatus:false];
    }else{
        
    }
    
    result(nil);
}

- (void)trackingStatus:(FlutterMethodCall*)call result:(FlutterResult)result{
    result(@([Apptics trackingStatus]));
}

- (void)setTrackingStatus:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *status = call.arguments[@"status"];
    [Apptics setTrackingStatus:status.boolValue];
    result(nil);
}

- (void)crashReportStatus:(FlutterMethodCall*)call result:(FlutterResult)result{
    result(@([Apptics crashReportStatus]));
}

- (void)setCrashReportStatus:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *status = call.arguments[@"status"];
    [Apptics setCrashReportStatus:status.boolValue];
    result(nil);
}

- (void)personalInfoTrackingStatus:(FlutterMethodCall*)call result:(FlutterResult)result{
    result(@([Apptics personalInfoTrackingStatus]));
}

- (void)setPersonalInfoTrackingStatus:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *status = call.arguments[@"status"];
    [Apptics setPersonalInfoTrackingStatus:status.boolValue];
    result(nil);
}

- (void)isUserLoggedIn:(FlutterMethodCall*)call result:(FlutterResult)result{
    result(@([Apptics isUserLoggedIn]));
}

- (void)setSendAnalyticsDataUsingMobileData:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *status = call.arguments[@"status"];
    [Apptics setSendAnalyticsDataUsingMobileData:status.boolValue];
    result(nil);
}

- (void)sendAnalyticsDataUsingMobileData:(FlutterMethodCall*)call result:(FlutterResult)result{
    result(@([Apptics sendAnalyticsDataUsingMobileData]));
}

- (void)getLocalizableStringForKey:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *key = call.arguments[@"key"];
    result([Apptics getLocalizableStringForKey:key]);
}

- (void)setDefaultLanguage:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *lang = call.arguments[@"lang"];
    [Apptics setDefaultLanguage:lang];
    result(nil);
}

- (void)setCurrentLocation:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *location = call.arguments[@"location"];
    [Apptics setCurrentLocation:location];
    result(nil);
}

- (void)setPrivacyConsentType:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *type = call.arguments[@"type"];
    [Apptics setPrivacyConsentType:type.intValue];
    result(nil);
}

- (void)setTypeOfAnonymity:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *type = call.arguments[@"type"];
    [Apptics setTypeOfAnonymity:type.intValue];
    result(nil);
}

- (void)enableAutoCheckForAppUpdate:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *status = call.arguments[@"status"];
    //    [Apptics enableAutoCheckForAppUpdate:status.boolValue];
    result(nil);
}

- (void)crash:(FlutterMethodCall*)call result:(FlutterResult)result{
    [Apptics crash];
    result(nil);
}

- (void)isReachable:(FlutterMethodCall*)call result:(FlutterResult)result{
    result(@([Apptics isReachable]));
}

- (void)addEvent:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *event = call.arguments[@"event"];
    NSString *group = call.arguments[@"group"];
    NSDictionary *custom_properties = call.arguments[@"properties"];
    [APEvent trackEvent:event andGroupName:group withProperties:custom_properties];
    result(nil);
}

- (void)trackViewEnter:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *screen_name = call.arguments[@"screen_name"];
    [APScreentracker trackViewEnter:screen_name];
    result(nil);
}

- (void)trackViewExit:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *screen_name = call.arguments[@"screen_name"];
    [APScreentracker trackViewExit:screen_name];
}

- (void) checkAndUpdateAlert:(FlutterMethodCall*)call result:(FlutterResult)result{
    
    [APAppUpdateManager checkForAppUpdate:^(NSDictionary * _Nullable updateInfo) {
        @try{
            
            NSDictionary *JSONData = updateInfo;
            
            if (JSONData == nil)
            {
                result(nil);
                return;
            }
            
            NSNumber *issupported = [JSONData objectForKey:@"issupported"];
            
            updateId = [JSONData objectForKey:@"updateid"];
            
            NSString *currentAppVersion = [JSONData objectForKey:@"currentversion"];
            
            if ([currentAppVersion isEqualToString:[ZASystemInfo getAppVersion]]){
                result(nil);
                return;
            }
            
            updateKey = [NSString stringWithFormat:@"%@", currentAppVersion];
            NSNumber *hasUpdate = [JSONData objectForKey:@"hasupdate"];
            
            if ([hasUpdate boolValue] == true){
                
                NSDictionary *updateData = [self getUpdateAlertData:JSONData];
                
                NSMutableDictionary *updatedetails = [[JSONData objectForKey:@"updatedetails"] mutableCopy];
                
                //                NSDictionary* localeInfo = [JSONData objectForKey:@"popupinfo"];
                //
                storeurl = [updatedetails objectForKey:@"storeurl"];
                storeurl = [storeurl stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
                reminder = [updatedetails objectForKey:@"reminder"];
                option = [updatedetails objectForKey:@"option"];
                
                if (option.intValue < 3){
                    if ([self shouldRemindForUpdate:updateKey] == false || [self shouldSkipThisUpdate:updateKey]){
                        result(nil);
                        return;
                    }
                }
                
                //                NSNumber *popuptype = [updatedetails objectForKey:@"popuptype"];
                //
                //                NSNumber *forceAfter = [updatedetails objectForKey:@"toforce"];
                //
                //                if ([self shouldConvertToForceForAnUpdate:updateKey forceAfter:forceAfter]){
                //                    option = @3;
                //                    updatedetails[@"option"] = option;
                //                }
                result([self convertUpdateDataToJson:updateData]);
                return;
                
            }else if ([issupported boolValue] == true){
                NSDictionary *notSupported = [self getNonSupportedAlertData:JSONData];
                if (notSupported != nil) {
                    result([self convertNonSupportAlertData:notSupported]);
                    return;
                } else {
                    result(nil);
                    return;
                }
            }else{
                [ZAKeyChainWrapper setNumber:updateId forKey:kJU_UpdateId];
                result(nil);
                return;
            }
            
            
            
            //              if ([issupported boolValue] == false){
            //
            //                if ([self shouldSkipThisUpdate:[@"-" stringByAppendingString:updateKey]]){
            //                    result(nil);
            //                  return;
            //                }
            //
            //                NSNumber *alerttype = [JSONData objectForKey:@"alerttype"];
            //                if (![alerttype isEqualToNumber:@(0)]){
            //
            //                  NSDictionary* localeInfo = [JSONData objectForKey:@"popupinfo"];
            //
            //                  NSString *featuretitle = AppticsLocalizedString( @"zanalytics.appupdates.title",[localeInfo objectForKey:@"title"]);
            //
            //                  NSString *description = AppticsLocalizedString( @"zanalytics.appupdates.description",[localeInfo objectForKey:@"description"]);
            //
            //                  description = [description stringByReplacingOccurrencesOfString:@"%osversion%" withString:[ZASystemInfo getOsVersion]];
            //                  NSString *continueTitle = [localeInfo objectForKey:@"installlater"];
            //                }
            //
            //              }else{}
            
        }@catch (NSException *exception){
            IVLog(@"exception in app update response handler %@", exception);
            result(nil);
            return;
        }
    }];
    
}

-(NSDictionary*) getUpdateAlertData: (NSDictionary *)jsonObject {
    if (jsonObject){
        NSDictionary *popupInfo = jsonObject[@"popupinfo"];
        NSDictionary *updateDetails = jsonObject[@"updatedetails"];
        return @{
            @"updateId" : jsonObject[@"updateid"],
            @"currentVersion" : jsonObject[@"currentversion"],
            @"featureTitle" : popupInfo[@"title"],
            @"features" : popupInfo[@"description"],
            @"remindMeLaterText" : (popupInfo[@"remindmelater"]) ? popupInfo[@"remindmelater"] : @"",
            @"updateNowText" : (popupInfo[@"updatenow"]) ? popupInfo[@"updatenow"] : @"",
            @"neverAgainText" : (popupInfo[@"ignore"]) ? popupInfo[@"ignore"] : @"",
            @"option" : updateDetails[@"option"],
            @"reminderDays" : (updateDetails[@"reminder"]) ? updateDetails[@"reminder"] : @"0",
            @"forceInDays" : updateDetails[@"toforce"],
            @"alertType" : updateDetails[@"popuptype"],
            @"customStoreUrl" : (updateDetails[@"storeurl"]) ? updateDetails[@"storeurl"] : @""
        };
        
    }
    return nil;
}

-(NSDictionary*) getNonSupportedAlertData: (NSDictionary *)jsonObject {
    if (jsonObject){
        NSDictionary *popupInfo = jsonObject[@"popupinfo"];
        if (popupInfo){
            return @{
                @"title" : popupInfo[@"title"],
                @"description" : popupInfo[@"description"],
                @"continueBtTxt" : popupInfo[@"installlater"],
                @"alertType" : jsonObject[@"alerttype"],
                @"updateId" : [NSString stringWithFormat:@"%@", jsonObject[@"updateid"]]
            };
        }
    }
    return nil;
}

-(NSDictionary*) convertUpdateDataToJson: (NSDictionary*) updateData{
    if (updateData != nil){
        return @{
            @"updateid" : [NSString stringWithFormat:@"%@", updateData[@"updateId"]],
            @"currentversion" : updateData[@"currentVersion"],
            @"featureTitle" : updateData[@"featureTitle"],
            @"features" : updateData[@"features"],
            @"remindMeLaterText" : updateData[@"remindMeLaterText"],
            @"updateNowText" : updateData[@"updateNowText"],
            @"neverAgainText" : updateData[@"neverAgainText"],
            @"option" : [NSString stringWithFormat:@"%@", updateData[@"option"]],
            @"reminderDays" : [NSString stringWithFormat:@"%@",updateData[@"reminderDays"]],
            @"forceInDays" : updateData[@"forceInDays"],
            @"alertType" : updateData[@"alertType"],
            @"customStoreUrl" : updateData[@"customStoreUrl"],
            @"category" : @1
        };
    }  return nil;
}

-(NSDictionary*) convertNonSupportAlertData:(NSDictionary*) appticsAppUpdateNotSupported {
    if (appticsAppUpdateNotSupported != nil) {
        return @{
            @"title" : appticsAppUpdateNotSupported[@"title"],
            @"description" : appticsAppUpdateNotSupported[@"description"],
            @"continueBtTxt" : appticsAppUpdateNotSupported[@"continueBtTxt"],
            @"alertType" : appticsAppUpdateNotSupported[@"alertType"],
            @"updateId" : appticsAppUpdateNotSupported[@"updateId"],
            @"category" : @2
        };
    } else
        return nil;
}

- (void) onSendImpressionStatus:(FlutterMethodCall*)call result:(FlutterResult)result{
    [self updateStatsWithAction:JUActionImpression AndUpdateId:updateId];
}

- (void) onClickUpdate:(FlutterMethodCall*)call result:(FlutterResult)result{
    [ZAKeyChainWrapper setNumber:updateId forKey:kJU_UpdateId];
    [self launchAppStore:storeurl];
    [self updateStatsWithAction:JUActionDownload AndUpdateId:updateId];
}

- (void) onClickReminder:(FlutterMethodCall*)call result:(FlutterResult)result{
    [self remindMeIn:reminder forUpdate:updateKey];
    [self updateStatsWithAction:JUActionLater AndUpdateId:updateId];
}

- (void) onClickIgnore:(FlutterMethodCall*)call result:(FlutterResult)result{
    [self skipThisUpdate:updateKey];
    [self updateStatsWithAction:JUActionIgnore AndUpdateId:updateId];
}

- (void) onClickNonSupportAlert:(FlutterMethodCall*)call result:(FlutterResult)result{
    [self skipThisUpdate:updateKey];
    [self updateStatsWithAction:JUActionIgnore AndUpdateId:updateId];
}

-(void) updateStatsWithAction : (NSString*) action AndUpdateId : (NSNumber*) updateid {

    if (updateid != nil){
        NSString *currentAppVersion = [ZASystemInfo getAppVersion];
        NSString *currentAppReleaseVersion = [ZASystemInfo getBuildNumber];
        [ZAAppupdatePopupInfo addObjectToQueueForDictionary:@{@"action":za_safeString(action), @"updateid":za_safeNumber(updateid), @"previousversion":za_safeString(currentAppVersion), @"appreleaseversion":za_safeString(currentAppReleaseVersion)}];
    }

}

-(void) launchAppStore : (NSString*) storeurl {
    dispatch_async(dispatch_get_main_queue(), ^{
        NSURL *iTunesURL = [NSURL URLWithString:storeurl];
        if (storeurl != nil && ![storeurl isEqual:@""] && iTunesURL != nil){

#if (TARGET_OS_IOS || TARGET_OS_TV || TARGET_OS_WATCH)
            [[ZAnalyticsUIApplication getInstance] za_openURL:iTunesURL options:nil completionHandler:^(BOOL success) {

            }];
#endif
        }else{
            [self connectToItunes];
        }
    });
}

-(void) connectToItunes{
    //Have to check if it is a development build
    
    NSString *bundleId = [ZASystemInfo getBundleID];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:[NSString stringWithFormat:@"lookup?bundleId=%@",bundleId] relativeToURL:[NSURL URLWithString:@"https://itunes.apple.com"]]];
    [request setHTTPMethod:@"GET"];
    [request setHTTPShouldHandleCookies:NO];

    NSURLSession *session = [NSURLSession sessionWithConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration] delegate:nil delegateQueue:nil];

    NSURLSessionDataTask *task = [session dataTaskWithRequest:request completionHandler:^(NSData * data, NSURLResponse * response, NSError * error) {

        if(error == nil)
        {
            NSError *jsonParsingError;
            NSDictionary *JSONData = [NSJSONSerialization JSONObjectWithData:data options:0 error:&jsonParsingError];

            if(jsonParsingError == nil)
            {
                NSHTTPURLResponse *httpResponse = (NSHTTPURLResponse*)response;
                if ([httpResponse statusCode] == 200)
                {
                    if ([[JSONData objectForKey:@"results"] isKindOfClass:[NSArray class]]){
                        NSArray *results = (NSArray*) [JSONData objectForKey:@"results"];
                        if ([results count] > 0){
                            NSDictionary *appInfo =  results.firstObject;
                            NSString *trackViewUrl = (NSString *)appInfo[@"trackViewUrl"];

                            if (trackViewUrl != nil) {
                                dispatch_async(dispatch_get_main_queue(), ^{
                                    NSURL *iTunesURL = [NSURL URLWithString:trackViewUrl];
#if (TARGET_OS_IOS || TARGET_OS_TV || TARGET_OS_WATCH)
                                    [[ZAnalyticsUIApplication getInstance] za_openURL:iTunesURL options:nil completionHandler:^(BOOL success) {

                                    }];
#endif

                                });
                            }
                        }
                    }
                }
            }
        }

    }];
    [task resume];
}

-(void) resetReminders{
    [NSUserDefaults removeObjectWithKey:JUDefaultReminderCheckDate];
    [NSUserDefaults removeObjectWithKey:JUDefaultRemindedCount];
}

-(void) remindMeIn : (NSNumber*) reminder forUpdate : (NSString*) updateKey{
    NSDate *today=[NSDate date];
    NSCalendar *calendar = [[NSCalendar alloc] initWithCalendarIdentifier: NSCalendarIdentifierGregorian];
    NSDateComponents *components=[[NSDateComponents alloc] init];
    components.day=reminder.integerValue;
    NSDate *targetDate =[calendar dateByAddingComponents:components toDate:today options: 0];

    [[NSUserDefaults standardUserDefaults] saveCustomObject:[NSDictionary dictionaryWithObjectsAndKeys:targetDate, updateKey, nil] key:JUDefaultReminderCheckDate];

    NSMutableDictionary *remindedInfo = [[[NSUserDefaults standardUserDefaults] loadCustomObjectWithKey:JUDefaultRemindedCount] mutableCopy];

    if (remindedInfo == nil){
        remindedInfo = [NSMutableDictionary new];
        remindedInfo[updateKey] = @1;
    }else{
        remindedInfo[updateKey] = @(za_safeNumber((NSNumber*)remindedInfo[updateKey]).integerValue
        + 1);
    }

    [[NSUserDefaults standardUserDefaults] saveCustomObject:remindedInfo key:JUDefaultRemindedCount];
    [self resetSkipUpdate];
}

-(BOOL) shouldConvertToForceForAnUpdate: (NSString*) updateKey forceAfter : (NSNumber*) daysCount {

    if (daysCount.intValue == 0)
        return false;

    NSDictionary *remindedInfo = [[NSUserDefaults standardUserDefaults] loadCustomObjectWithKey:JUDefaultRemindedCount] ;
    if (remindedInfo == nil){
        return false;
    }else{
        return (za_safeNumber((NSNumber*)remindedInfo[updateKey]).intValue >= daysCount.intValue);
    }
}

-(BOOL) shouldRemindForUpdate : (NSString*) updateKey {

    NSDictionary *reminderInfo = [[NSUserDefaults standardUserDefaults] loadCustomObjectWithKey:JUDefaultReminderCheckDate];
    id date = [reminderInfo objectForKey:updateKey];
    if (date != nil) {
        NSDate *currentDate = [NSDate date];
        NSDate *updateDate = (NSDate*) date;

        if ([currentDate compare:updateDate] == NSOrderedAscending || [currentDate compare:updateDate] == NSOrderedSame) {
            return false;
        }else{
            return true;
        }
    }
    return true;
}

-(void) resetSkipUpdate{
    [NSUserDefaults removeObjectWithKey:kJU_SkipThisAppVersion];
}

-(BOOL) shouldSkipThisUpdate : (NSString*) updateKey{

    NSDictionary *skipInfo = [[NSUserDefaults standardUserDefaults] loadCustomObjectWithKey:kJU_SkipThisAppVersion];
    NSNumber *shouldSkip = za_safeNumber((NSNumber*)[skipInfo objectForKey:updateKey]);
    return shouldSkip.boolValue;

}

-(void) skipThisUpdate : (NSString*) updateKey{
  [[NSUserDefaults standardUserDefaults] saveCustomObject:[NSDictionary dictionaryWithObjectsAndKeys:@1, za_safeString(updateKey), nil] key:kJU_SkipThisAppVersion];
  [self resetReminders];
}

- (void)navigationController:(UINavigationController *)navigationController willShowViewController:(UIViewController *)viewController animated:(BOOL)animated{
    if (childviewcontrollers > 1){
        double delayInSeconds = 0.5;
        dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
        dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
            [window setHidden:true];
        });
    }
    childviewcontrollers = navigationController.childViewControllers.count;
}

#pragma Rateus methods

-(void)checkForRatingPop:(FlutterMethodCall*)call result:(FlutterResult)result{
    dispatch_async(dispatch_get_main_queue(), ^{
        _result = result;
        [APRateUs checkForRatingPop];
    });
}

-(void)getCriteriaId:(FlutterMethodCall*)call result:(FlutterResult)result{
    [self checkForRatingPop:call result:result];
}

-(void)isAppticsFeedbackModuleAvailable:(FlutterMethodCall*)call result:(FlutterResult)result{
#ifdef __HAS_APPTICS_FEEDBACKKIT_FRAMEWORK__
    result(@(YES));
#else
    result(@(NO));
#endif
}

-(void)openFeedback:(FlutterMethodCall*)call result:(FlutterResult)result{
#ifdef __HAS_APPTICS_FEEDBACKKIT_FRAMEWORK__
    [FeedbackKit showFeedback];
    result(nil);
#endif
}

-(void)openPlayStore:(FlutterMethodCall*)call result:(FlutterResult)result{
    [APRateUs takeToAppStoreReviewScreen];
}

-(void)daysBeforeShowingPopupAgain:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *promptedDateString = [ZAKeyChainWrapper stringForKey:kJA_autoPromptedDate];
    NSNumber *daysCount = @0;
    if (promptedDateString){
        NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
        [dateFormatter setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"en_US_POSIX"]];
        [dateFormatter setDateFormat:@"dd-MM-yyyy"];
        NSDate *promptedDate = [dateFormatter dateFromString:promptedDateString];
        NSTimeInterval secondsBetween = [[NSDate date] timeIntervalSinceDate:promptedDate];
        daysCount = @((int)(secondsBetween / 86400));
    }
    result(daysCount);
}

-(void)maxTimesToShowPopup:(FlutterMethodCall*)call result:(FlutterResult)result{
    result(maxTimesToShowPopup);
}

-(void)showStoreAlertOnFulFillingCriteria:(FlutterMethodCall*)call result:(FlutterResult)result{
    result(@(showStorePromptOnFulFillingCriteria));
}

-(void)disableIfNotInstalledFromPlayStore:(FlutterMethodCall*)call result:(FlutterResult)result{
    result(@(disableIfNotInstalledFromAppStore));
}

-(void)setDaysBeforeShowingPopupAgain:(FlutterMethodCall*)call result:(FlutterResult)result{
    result(nil);
}

-(void)setMaxTimesToShowPopup:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *status = call.arguments[@"maxTime"];
    maxTimesToShowPopup = status;
    result(nil);
}

-(void)setShowStoreAlertOnFulFillingCriteria:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *status = call.arguments[@"isShow"];
    showStorePromptOnFulFillingCriteria=status.boolValue;
    [APRateUs showStoreAlertOnFulFillingCriteria:status.boolValue];
    result(nil);
}

-(void)setDisableIfNotInstalledFromPlayStore:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *status = call.arguments[@"ifNotInstalledFromPlayStore"];
    disableIfNotInstalledFromAppStore=status.boolValue;
    result(nil);
}

-(void)sentStats:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSNumber *popupAction = call.arguments[@"popupAction"];
    switch (popupAction.intValue) {
        case 1:
            [[APStoreReviewManager getInstance] updateRateUAction:JRateUsActionRateOnStore type:JRateUsTypePopup1 source:JRateUsSourceManual];
            break;
        case 0:
            [[APStoreReviewManager getInstance] updateRateUAction:JRateUsActionSendFeedback type:JRateUsTypePopup1 source:JRateUsSourceManual];
            break;
        case -1:
            [[APStoreReviewManager getInstance] updateRateUAction:JRateUsActionCancel type:JRateUsTypePopup1 source:JRateUsSourceManual];
            break;
    }

}

-(void)updateRatingShown:(FlutterMethodCall*)call result:(FlutterResult)result{
    [[APStoreReviewManager getInstance] setAppRatingShown];
}

#pragma Remote Config Methods

-(void) hardReset:(FlutterMethodCall*)call result:(FlutterResult)result{

    self.lastFetchedTime = 0;
}

-(void) getStringValue:(FlutterMethodCall*)call result:(FlutterResult)result{
    bool fallbackWithOfflineValue = [(NSNumber*) call.arguments[@"fallbackWithOfflineValue"] boolValue];
    bool coldFetch = [(NSNumber*) call.arguments[@"coldFetch"] boolValue];

    NSTimeInterval currentTime = [[NSDate date] timeIntervalSince1970];
    NSTimeInterval timeDifference = currentTime - self.lastFetchedTime;

    if (coldFetch){
        if (self.lastFetchedTime == 0 || timeDifference > 10){

            self.lastFetchedTime = [[NSDate date] timeIntervalSince1970];

            [[APRemoteConfig shared] fetchRemoteConfigWithCompletion:^(APRemoteConfigFetchStatus status) {
                if (status == APRemoteConfigFetchStatusSuccess || status == APRemoteConfigFetchStatusUpToDate){
                    if ([[APRemoteConfig shared] activateFetched] == true){
                        [self getValue:call result:result];
                    }else{
                        IVLog(@"Data is empty");
                        result(nil);
                    }
                }else if (status == APRemoteConfigFetchStatusFailure){
                    IVLog(@"Failed to fetch data from the server");
                    result(nil);
                }
            }];
        }else if (fallbackWithOfflineValue){
            if ([APRemoteConfig shared].lastFetchStatus != APRemoteConfigFetchStatusFailure){
                [self getValue:call result:result];
            }else{
                IVLog(@"Failed to fetch data from the server");
                result(nil);
            }
        }else{
            IVLog(@"Last fetch happened before %i sec, try after %i sec", timeDifference, (10 - timeDifference));
            result(nil);
        }
    }else{
        if (self.lastFetchedTime == 0){

            self.lastFetchedTime = [[NSDate date] timeIntervalSince1970];

            [[APRemoteConfig shared] fetchRemoteConfigWithCompletion:^(APRemoteConfigFetchStatus status) {
                if (status == APRemoteConfigFetchStatusSuccess || status == APRemoteConfigFetchStatusUpToDate){
                    if ([[APRemoteConfig shared] activateFetched] == true){
                        [self getValue:call result:result];
                    }else{
                        IVLog(@"Data is empty");
                        result(nil);
                    }
                }else if (status == APRemoteConfigFetchStatusFailure){
                    IVLog(@"Failed to fetch data from the server");
                    result(nil);
                }
            }];
        }else{
            if ([APRemoteConfig shared].lastFetchStatus == APRemoteConfigFetchStatusFailure){
                IVLog(@"Failed to fetch data from the server");
                result(nil);
            }else if ([APRemoteConfig shared].lastFetchStatus == APRemoteConfigFetchStatusNotYetFetched){
                IVLog(@"Fetching data from server...");
                result(nil);
            }else{
                [self getValue:call result:result];
            }
        }
    }

}

-(void) getValue:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *key = call.arguments[@"key"];
    APRemoteConfigValue *value = [APRemoteConfig shared][key];
    if (value != nil && value.stringValue != nil){
        result(value.stringValue);
    }else{
        IVLog(@"Value not found for the key '%@'", key);
        result(nil);
    }
}

-(void) setCustomConditionValue:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *key = call.arguments[@"key"];
    NSString *value = call.arguments[@"value"];
    [[APRemoteConfig shared] addCustomCriteria:key value:value];
    result(nil);
}


- (void) crashTracker:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *issueName = call.arguments[@"issueName"];
    NSString *stackTrace = call.arguments[@"stackTrace"];
    NSString *arch = call.arguments[@"arch"];
    NSString *reason = call.arguments[@"reason"];
    NSNumber *isFatal = call.arguments[@"isFatal"];
    if (isFatal.boolValue == true){
        [[Analytics getInstance] saveFLFatals:issueName message:reason stackTrace:stackTrace crashProperties:[self crashProperties]];
    }else{
        [[Analytics getInstance] saveFLNonFatalsUniqueId:[NSString stringWithFormat:@"non-fatal-%@",@([ZASystemInfo getTimeinLong])] issueName:issueName message:reason stackTrace:stackTrace crashProperties:[self crashProperties]];
    }
    result(@true);
}

- (NSMutableDictionary*) crashProperties
{
  ZASystemProperties *properties = [ZASystemProperties sharedInstance];
    NSDictionary *_crashProperties = @{
        @"networkstatus": za_safeNumber(properties.networkReachableStatus),
        @"serviceprovider": @0,
        @"orientation": za_safeNumber(properties.phoneOrientation),
        @"batterystatus": za_safeNumber(properties.batterylevel),
        @"edge": za_safeString(properties.currentRadioAccess),
        @"ram": za_safeString(properties.ram),
        @"rom": za_safeString(properties.rom),
        @"sessionstarttime": za_safeNumber([ZAGlobalQueue sessionStartTime]),
        @"screenname": za_safeString([ZAGlobalQueue currentScreenName]),
        @"happenedat" : @([ZASystemInfo getTimeinLong]),
        @"customproperties" : rt_safeDictionary(customproperties)
    };
    return _crashProperties.mutableCopy;
}
@end

@implementation AppticsCrashFlutterPlugin

+(AppticsCrashFlutterPlugin*) shared{

    static dispatch_once_t onceToken;
    __strong static id _sharedObject = nil;
    dispatch_once(&onceToken, ^{
        _sharedObject = [[AppticsCrashFlutterPlugin alloc] init];

        [[NSNotificationCenter defaultCenter] addObserver:_sharedObject selector:@selector(checkForCrashAndSendToServer:) name:@"APCheckForCrashAndSendToServerNotification" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:_sharedObject selector:@selector(uploadCrashSuccessNotification:) name:@"APFLTCrashUploadSuccessNotification" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:_sharedObject selector:@selector(uploadNonFatalsSuccessNotification:) name:@"APFLTNonFatalsUploadSuccessNotification" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:_sharedObject selector:@selector(applicationWillEnterForeground:) name:UIApplicationWillEnterForegroundNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:_sharedObject selector:@selector(applicationDidEnterBackground:) name:UIApplicationDidEnterBackgroundNotification object:nil];
//        NSSetUncaughtExceptionHandler(&uncaughtExceptionHandler);
    });
    return _sharedObject;
}

- (void) applicationWillEnterForeground:(NSNotification*) notification
{
    [AppticsCrashFlutterPlugin shared].uniqErrors=[NSMutableDictionary new];
}

- (void) applicationDidEnterBackground:(NSNotification*) notification
{
    [self saveUniqueNonFatalDetails];
}

//+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
//
//    [Apptics initializeWithVerbose:true];
//
//    FlutterMethodChannel* channel = [FlutterMethodChannel
//                                     methodChannelWithName:@"apptics_flutter_crash"
//                                     binaryMessenger:[registrar messenger]];
//    AppticsCrashFlutterPlugin* instance = [[AppticsCrashFlutterPlugin alloc] init];
//
//    [registrar addMethodCallDelegate:instance channel:channel];
//}

-(void) saveUniqueNonFatalDetails{
    if ([ZAGlobalQueue sessionStartTime] != nil && [AppticsCrashFlutterPlugin shared].uniqErrors.count > 0){
        NSString *crashFileName = [NSString stringWithFormat:@"%@/../FLNonFatals-meta/%@.plist", [ZAFileManager dirPathForCrash], [ZAGlobalQueue sessionStartTime]];
        [ZAFileManager archiveFLCrashReport:[AppticsCrashFlutterPlugin shared].uniqErrors withFileName:crashFileName];
    }
}

-(void) uploadCrashSuccessNotification : (NSNotification*) notification{
    NSArray *files = (NSArray*) notification.object;
    if (files && files.count > 0){
        for (NSString *file in files){
            NSString *filePath = [[[ZAFileManager dirPathForCrash] stringByAppendingString:@"/../FLFatals"] stringByAppendingPathComponent:file];
            if (filePath){
                [ZAFileManager removeFileFromPath:filePath];
            }
        }
    }

    [self checkForNonFatalAndSendToServer];
}

-(void) uploadNonFatalsSuccessNotification : (NSNotification*) notification{
    NSArray *files = (NSArray*) notification.object;
    if (files && files.count > 0){
        for (NSString *file in files){
            NSString *filePath = [[[ZAFileManager dirPathForCrash] stringByAppendingString:@"/../FLNonFatals"] stringByAppendingPathComponent:file];
            if (filePath){
                NSMutableDictionary *crash = [ZAFileManager unarchiveFLCrashReportForFileName:filePath];
                NSDictionary *exceptiondetails = crash[@"exceptiondetails"];
                NSNumber *sessionStartTime=exceptiondetails[@"sessionstarttime"];
                [ZAFileManager removeFileFromPath:filePath];

                NSString *metaFilePath = [[ZAFileManager dirPathForCrash] stringByAppendingFormat:@"/../FLNonFatals-meta/%@.plist",sessionStartTime];
                [ZAFileManager removeFileFromPath:metaFilePath];
            }
        }
    }
}

-(void) checkForCrashAndSendToServer : (NSNotification*) notification{
    NSInteger crashCount = [self crashReportCount];
    if (crashCount > 0){
        if ([Analytics getInstance].enableAutomaticCrashTracking){
            //Ask for user's permission to send crash report when tracking is completely off
            if ([[ZAPIIManager getInstance] shouldSendCrashReport]){
                [self sendPendingCrashReports];
            }else{
                if ([[ZAPIIManager getInstance] shouldAskPermissionForCrashReport]){
                    [[ZAPIIManager getInstance] askPermissionToSendCrashReport:^(CARType type) {
                        if (type == CARTypeSend){
                            [self sendPendingCrashReports];
                        }else{
                            //remove crashes
                            [self deleteAllCrashReports];
                        }
                    }];
                }else{
                    //remove crashes
                    [self deleteAllCrashReports];
                }
            }
        }else{
            //remove crashes
            [self deleteAllCrashReports];
        }
    }else{
        //Check for non-fatals and send
        [self checkForNonFatalAndSendToServer];
    }
}

- (NSUInteger) crashReportCount {
    NSArray *files = [ZAFileManager getFLCrashes];
    return files.count;
}

- (void) sendPendingCrashReports{
    NSArray *files = [ZAFileManager getFLCrashes];
    if (files.count > 0){

        [[ZAPIIManager getInstance] newRegisterDevice:^(NSString* deviceId) {
            NSString *dirPath;

            dirPath = [[ZAFileManager dirPathForCrash] stringByAppendingString:@"/../FLFatals"];

            NSFileManager *fm = [NSFileManager defaultManager];

            __block NSDictionary *meta;
            NSMutableArray *crashInfos = [NSMutableArray new];
            NSMutableArray *reportIds = [NSMutableArray new];
            __block NSUInteger datasize=0;

            [files enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
                NSString *crashFile = (NSString*) obj;
                NSString *filePath = [dirPath stringByAppendingPathComponent:crashFile];
                if ([fm fileExistsAtPath:filePath]){
                    NSMutableDictionary *crash = [ZAFileManager unarchiveFLCrashReportForFileName:filePath];
                    NSDictionary *_meta = crash[@"meta"];
                    NSDictionary *exceptiondetails = crash[@"exceptiondetails"];
                    if (exceptiondetails){
                        NSNumber *size = crash[@"size"];
                        datasize += size.integerValue;
                        if (meta && ![meta isEqualToDictionary:_meta]){
                            *stop = true;
                            return;
                        }
                        //                            IVLog(@"Size is : %.2f",(float)datasize/1024.0f/1024.0f);
                        if (((float)datasize/1024.0f/1024.0f) >= maxDataSize)
                        {
                            *stop = true;
                            return;
                        }

                        meta = _meta;
                        [crashInfos addObject:exceptiondetails];
                        [reportIds addObject:crashFile];
                        NSLog(@"Updated crashInfos :: %@",crashInfos);
                    }
                }
            }];
            if (meta == nil){
                meta = [ZASystemProperties appMeta];
            }

            NSString *userId = @"";
            NSString *anonid = @"";

            if ([[ZAPIIManager getInstance] isAnonymousUser]){
                //unknown or unauthorised or guest user.
                userId = @"";
                if ([Analytics getInstance].enableAnonymousTracking == true && [ZAPIIManager getInstance].anonid)
                    anonid = [ZAPIIManager getInstance].anonid;
            }else if ([[ZAPIIManager getInstance] isUserLoggedIn] && [[Analytics getInstance] getCurrentUserId]){
                userId = [[Analytics getInstance] getCurrentUserId];
            }

            if (meta && crashInfos.count > 0){
                APCrashReportUploadOperation *uploadOperation = [[APCrashReportUploadOperation alloc] initWithCrashReport:@{@"meta":meta,@"exceptiondetails":crashInfos,@"reportIDs":reportIds,@"userid":userId, @"anonid":anonid, @"isfatal":@true}];
                uploadOperation.url = [self.crashUrl URLByAppendingPathComponent:@"fatal"];
                [[ZAGlobalQueue sharedQueue].operationQueue addOperation:uploadOperation];
            }
        }];
    }
}

- (NSUInteger) nonFatalsReportCount {
    NSArray *files = [ZAFileManager getFLNonFatals];
    return files.count;
}

- (void) sendPendingNonFatalsReports{
    NSArray *files = [ZAFileManager getFLNonFatals];
    if (files.count > 0){

        [[ZAPIIManager getInstance] newRegisterDevice:^(NSString* deviceId) {
            NSString *dirPath;

            dirPath = [[ZAFileManager dirPathForCrash] stringByAppendingString:@"/../FLNonFatals"];

            NSFileManager *fm = [NSFileManager defaultManager];

            __block NSDictionary *meta;
            NSMutableArray *crashInfos = [NSMutableArray new];
            NSMutableArray *reportIds = [NSMutableArray new];
            __block NSUInteger datasize=0;

            [files enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
                NSString *crashFile = (NSString*) obj;
                NSString *filePath = [dirPath stringByAppendingPathComponent:crashFile];
                if ([fm fileExistsAtPath:filePath]){
                    NSMutableDictionary *crash = [ZAFileManager unarchiveFLCrashReportForFileName:filePath];
                    NSDictionary *_meta = crash[@"meta"];
                    NSMutableDictionary *exceptiondetails = ((NSDictionary*)crash[@"exceptiondetails"]).mutableCopy;
                    if (exceptiondetails){
                        NSNumber *size = crash[@"size"];
                        datasize += size.integerValue;
                        if (meta && ![meta isEqualToDictionary:_meta]){
                            *stop = true;
                            return;
                        }
                        //                            IVLog(@"Size is : %.2f",(float)datasize/1024.0f/1024.0f);
                        if (((float)datasize/1024.0f/1024.0f) >= maxDataSize)
                        {
                            *stop = true;
                            return;
                        }

                        meta = _meta;
                        NSString *uniqueid = crash[@"uniqueid"];
                        if (uniqueid != nil && ![uniqueid isEqual:@""]){
                            NSNumber *sessionStartTime=exceptiondetails[@"sessionstarttime"];
                            NSString *filePath = [[ZAFileManager dirPathForCrash] stringByAppendingFormat:@"/../FLNonFatals-meta/%@.plist",sessionStartTime];
                            NSDictionary *listofhappenedat = [ZAFileManager unarchiveFLCrashReportForFileName:filePath];
                            NSArray *happened_at=listofhappenedat[uniqueid];
                            if (happened_at != nil && happened_at.count > 0){
                                [exceptiondetails setObject:[happened_at componentsJoinedByString:@","] forKey:@"listofhappenedtime"];
                                [exceptiondetails setObject:@(happened_at.count) forKey:@"happenedcount"];
                            }
                        }
                        [crashInfos addObject:exceptiondetails];
                        [reportIds addObject:crashFile];
                        NSLog(@"Updated crashInfos :: %@",crashInfos);
                    }
                }
            }];
            if (meta == nil){
                meta = [ZASystemProperties appMeta];
            }

            NSString *userId = @"";
            NSString *anonid = @"";

            if ([[ZAPIIManager getInstance] isAnonymousUser]){
                //unknown or unauthorised or guest user.
                userId = @"";
                if ([Analytics getInstance].enableAnonymousTracking == true && [ZAPIIManager getInstance].anonid)
                    anonid = [ZAPIIManager getInstance].anonid;
            }else if ([[ZAPIIManager getInstance] isUserLoggedIn] && [[Analytics getInstance] getCurrentUserId]){
                userId = [[Analytics getInstance] getCurrentUserId];
            }

            if (meta && crashInfos.count > 0){
                APCrashReportUploadOperation *uploadOperation = [[APCrashReportUploadOperation alloc] initWithCrashReport:@{@"meta":meta,@"exceptiondetails":crashInfos,@"reportIDs":reportIds,@"userid":userId, @"anonid":anonid, @"isfatal":@false}];
                uploadOperation.url = [self.crashUrl URLByAppendingPathComponent:@"nonfatal"];
                [[ZAGlobalQueue sharedQueue].operationQueue addOperation:uploadOperation];
            }
        }];
    }
}

-(NSURL*) crashUrl {

    NSString *urlString = [NSString stringWithFormat:@"%@/hsdk/apptics/v1/exception",[[ZANetworking sharedManager] getBaseUrlForUserID:[ZAPIIManager getInstance].mam]];

    return [NSURL URLWithString:urlString];
}

- (void) deleteAllCrashReports{
    [ZAFileManager deleteAllFLCrashes];
    [self checkForNonFatalAndSendToServer];
}

- (void) checkForNonFatalAndSendToServer{
    NSInteger crashCount = [self nonFatalsReportCount];
    if (crashCount > 0){
        if ([[ZAPIIManager getInstance] shouldSendCrashReport]){
            [self sendPendingNonFatalsReports];
        }else{
            [ZAFileManager deleteAllFLNonFatals];
            [ZAFileManager deleteAllFLNonFatalsMeta];
        }
    }
}

@end

@implementation Analytics (AFLExtension)

+(void) load{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^
                  {
        double delayInSeconds = 3;
        dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
        dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
            [AppticsCrashFlutterPlugin shared].uniqErrors=[NSMutableDictionary new];
        });
    });
}

- (void) saveFLFatals : (NSString *)issueName message : (NSString *)message stackTrace : (NSString*)stackTrace crashProperties : (NSMutableDictionary*) crashProperties;{
    NSString *appName;
    appName = [ZASystemInfo getAppName];
    [ZAFileManager createCrashDir:@"/../FLFatals"];
    NSString *crashFileName = [NSString stringWithFormat:@"%@/../FLFatals/%@-report-%@.plist", [ZAFileManager dirPathForCrash], appName, [ZASystemInfo getTime]];
    NSMutableDictionary *crash = [NSMutableDictionary new];
    NSDictionary *meta = [ZASystemProperties appMeta];
    if (meta){
        [crash setObject:meta forKey:@"meta"];
    }
    [crashProperties setObject:issueName forKey:@"issuename"];
    [crashProperties setObject:stackTrace forKey:@"message"];
    [crashProperties setObject:@"flutter" forKey:@"errortype"];
    [crashProperties setObject:@1 forKey:@"happenedcount"];
    [crashProperties setObject:[NSString stringWithFormat:@"%@",[crashProperties objectForKey:@"happenedat"]] forKey:@"listofhappenedtime"];
    [crash setObject:crashProperties forKey:@"exceptiondetails"];
    NSError *error;
    NSData *crashInfoData = [NSJSONSerialization dataWithJSONObject:crashProperties options:1 error:&error];
    [crash setObject:@(crashInfoData.length) forKey:@"size"];

    [ZAFileManager archiveFLCrashReport:crash withFileName:crashFileName];

}

- (void) saveFLNonFatalsUniqueId : (NSString*) uniqueId issueName : (NSString *)issueName message : (NSString *)message stackTrace : (NSString*)stackTrace crashProperties : (NSMutableDictionary*) crashProperties{
    if (![[AppticsCrashFlutterPlugin shared].uniqErrors.allKeys containsObject:uniqueId]){
        NSString *appName;
        appName = [ZASystemInfo getAppName];
        [ZAFileManager createCrashDir:@"/../FLNonFatals"];
        [ZAFileManager createCrashDir:@"/../FLNonFatals-meta"];
        NSString *crashFileName = [NSString stringWithFormat:@"%@/../FLNonFatals/%@-report-%@.plist", [ZAFileManager dirPathForCrash], appName, [ZASystemInfo getTime]];
        NSMutableDictionary *crash = [NSMutableDictionary new];
        NSDictionary *meta = [ZASystemProperties appMeta];
        if (meta){
            [crash setObject:meta forKey:@"meta"];
        }
        [crashProperties setObject:issueName forKey:@"issuename"];
        [crashProperties setObject:stackTrace forKey:@"message"];
        [crashProperties setObject:@"flutter" forKey:@"errortype"];
        [crashProperties setObject:@1 forKey:@"happenedcount"];
        [crashProperties setObject:[NSString stringWithFormat:@"%@",[crashProperties objectForKey:@"happenedat"]] forKey:@"listofhappenedtime"];
        [crash setObject:crashProperties forKey:@"exceptiondetails"];
        NSError *error;
        NSData *crashInfoData = [NSJSONSerialization dataWithJSONObject:crashProperties options:1 error:&error];
        [crash setObject:@(crashInfoData.length) forKey:@"size"];
        [crash setObject:uniqueId forKey:@"uniqueid"];

        [ZAFileManager archiveFLCrashReport:crash withFileName:crashFileName];
        [[AppticsCrashFlutterPlugin shared].uniqErrors setObject:@[[ZASystemInfo getTime]].mutableCopy forKey:uniqueId];
    }else{
        NSMutableArray *happened_at=[[AppticsCrashFlutterPlugin shared].uniqErrors objectForKey:uniqueId];
        if (happened_at.count <= 10){
            [happened_at addObject:[ZASystemInfo getTime]];
            [[AppticsCrashFlutterPlugin shared].uniqErrors setObject:happened_at forKey:uniqueId];
        }
    }
}

@end

@implementation ZAFileManager (AFLExtension)

+ (void) archiveFLCrashReport:(NSDictionary*)crashReport withFileName : (NSString*) filePath{

      NSDictionary *copyOfData = [NSDictionary dictionaryWithDictionary:[crashReport copy]];
      if (![NSKeyedArchiver archiveRootObject:copyOfData toFile:filePath])
      {

      }
}

+(id) unarchiveFLCrashReportForFileName : (NSString*) filePath
{
  id unarchivedData = nil;
  @try {
    unarchivedData = [NSKeyedUnarchiver unarchiveObjectWithFile:filePath];
  }
  @catch (NSException *exception) {
    NSLog(@"%@ unable to unarchive crash report in %@, starting fresh", self, filePath);
    unarchivedData = nil;
  }

  return unarchivedData;

}

+(NSArray*) getFLCrashes{
    NSError *error;
  NSString *dirPath = [[[self class] dirPathForCrash] stringByAppendingString:@"/../FLFatals"];
    dirPath = [dirPath stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]];
  NSArray *dirs = [[NSFileManager defaultManager] contentsOfDirectoryAtURL:[NSURL URLWithString:dirPath]
                                                includingPropertiesForKeys:@[NSURLContentModificationDateKey]
                                                                   options:NSDirectoryEnumerationSkipsHiddenFiles
                                                                     error:&error];
    if (error == nil){
        NSMutableArray *files = [[NSMutableArray alloc] init];
        [dirs enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
          NSString *filename = (NSString *)obj;
            [files addObject:filename.lastPathComponent];
        }];

        return files;
    }else{
        return @[];
    }

}

+ (void) deleteAllFLCrashes{

    NSArray *files = [ZAFileManager getFLCrashes];
    NSString *dirPath;
    dirPath = [[[self class] dirPathForCrash] stringByAppendingString:@"/../FLFatals"];
    NSFileManager *fm = [NSFileManager defaultManager];

    [files enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        NSString *crashFile = (NSString*) obj;
        NSString *filePath = [dirPath stringByAppendingPathComponent:crashFile];
        if ([fm fileExistsAtPath:filePath]){
            [ZAFileManager removeFileFromPath:filePath];
        }
    }];

}

+(NSArray*) getFLNonFatals{
    NSError *error;
  NSString *dirPath = [[[self class] dirPathForCrash] stringByAppendingString:@"/../FLNonFatals"];
    dirPath = [dirPath stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]];
  NSArray *dirs = [[NSFileManager defaultManager] contentsOfDirectoryAtURL:[NSURL URLWithString:dirPath]
                                                includingPropertiesForKeys:@[NSURLContentModificationDateKey]
                                                                   options:NSDirectoryEnumerationSkipsHiddenFiles
                                                                     error:&error];
    if (error == nil){
        NSMutableArray *files = [[NSMutableArray alloc] init];
        [dirs enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
          NSString *filename = (NSString *)obj;
            [files addObject:filename.lastPathComponent];
        }];

        return files;
    }else{
        return @[];
    }

}

+ (void) deleteAllFLNonFatals{

    NSArray *files = [ZAFileManager getFLNonFatals];
    NSString *dirPath;
    dirPath = [[[self class] dirPathForCrash] stringByAppendingString:@"/../FLNonFatals"];
    NSFileManager *fm = [NSFileManager defaultManager];

    [files enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        NSString *crashFile = (NSString*) obj;
        NSString *filePath = [dirPath stringByAppendingPathComponent:crashFile];
        if ([fm fileExistsAtPath:filePath]){
            [ZAFileManager removeFileFromPath:filePath];
        }
    }];

}

+(NSArray*) getFLNonFatalsMeta{
    NSError *error;
  NSString *dirPath = [[[self class] dirPathForCrash] stringByAppendingString:@"/../FLNonFatals-meta"];
    dirPath = [dirPath stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]];
  NSArray *dirs = [[NSFileManager defaultManager] contentsOfDirectoryAtURL:[NSURL URLWithString:dirPath]
                                                includingPropertiesForKeys:@[NSURLContentModificationDateKey]
                                                                   options:NSDirectoryEnumerationSkipsHiddenFiles
                                                                     error:&error];
    if (error == nil){
        NSMutableArray *files = [[NSMutableArray alloc] init];
        [dirs enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
          NSString *filename = (NSString *)obj;
            [files addObject:filename.lastPathComponent];
        }];

        return files;
    }else{
        return @[];
    }

}

+ (void) deleteAllFLNonFatalsMeta{

    NSArray *files = [ZAFileManager getFLNonFatalsMeta];
    NSString *dirPath;
    dirPath = [[[self class] dirPathForCrash] stringByAppendingString:@"/../FLNonFatals-meta"];
    NSFileManager *fm = [NSFileManager defaultManager];

    [files enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        NSString *crashFile = (NSString*) obj;
        NSString *filePath = [dirPath stringByAppendingPathComponent:crashFile];
        if ([fm fileExistsAtPath:filePath]){
            [ZAFileManager removeFileFromPath:filePath];
        }
    }];

}

@end

