import 'package:flutter/src/widgets/framework.dart';

import 'apptics_flutter_platform_interface.dart';
import 'apptics_flutter_util.dart';


class AppticsFlutter {

static AppticsFlutter? _instance;

  static AppticsFlutter get instance {
    _instance ??= AppticsFlutter();
    return _instance!;
  }


  ///  This method will show the user the privacy options that are built into the Apptics SDK.

  Future<void> presentPrivacyReviewPopup() {
    return AppticsFlutterPlatform.instance.presentPrivacyReviewPopup();
  }

  /// This method opens the screen where the user can change their analytic privacy settings.

  Future<void> openPrivacySettings() {
    return AppticsFlutterPlatform.instance.openPrivacySettings();
  }

  /// Specify a user identifier which will be used to identify the user in the future. We recommend you use a unique userID.

  /// - Warning:
  /// To protect users privacy, please do not pass any personally identifiable information to this method.

  /// @param userId String

  Future<void> setUser(String userId) {
    return AppticsFlutterPlatform.instance.setUser(userId);
  }

  /// Returns the enum TrackingState for the current user:
  /// noTracking,
  /// usageAndCrashTrackingWithPII,
  /// onlyUsageTrackingWithPII,
  /// onlyCrashTrackingWithPII,
  /// usageAndCrashTrackingWithoutPII,
  /// onlyUsageTrackingWithoutPII,
  /// onlyCrashTrackingWithoutPII
  /// @return TrackingState

  Future<TrackingState?> getTrackingState() {
    return AppticsFlutterPlatform.instance.getTrackingState();
  }

  /// Set tracking state for the current user.

  /// @param state TrackingState

  Future<void> setTrackingState(TrackingState state) {
    return AppticsFlutterPlatform.instance.setTrackingState(state);
  }

  /// Returns the logged-in status of the user associated with the device.

  /// @return true if logged in else false.

  Future<bool?> isUserLoggedIn() {
    return AppticsFlutterPlatform.instance.isUserLoggedIn();
  }

  /// Use this method to track an event. Make sure you give meaningful names for events and their groups

  /// @param eventName name of the Occuring Event
  /// @param eventGroup name of the Group
  /// @param properties any object that is conforms to JSON Encoding

  Future<void> addEvent(
       String? event,
      String? group,
      {Map<String, dynamic>? properties}) {
    return AppticsFlutterPlatform.instance.addEvent(event, group, properties);
  }

  /// Use this method to track the attached screen.

  /// @param screen_name String (Enter the name of the screen you want to track)

  Future<void> screenAttached(String screen_name) {
    return AppticsFlutterPlatform.instance.screenAttached(screen_name);
  }

  /// Use this method to track the screen that is being detached.

  /// @param screen_name String (Enter the name of the screen you want to track)

  Future<void> screenDetached(String screen_name) {
    return AppticsFlutterPlatform.instance.screenDetached(screen_name);
  }

  /// Use this method to set the language for the Analytics SDK. If the language is not found in the resource bundle, the default language will be selected.
  /// @param lang (code) String.

  Future<void> setDefaultLanguage(String lang) {
    return AppticsFlutterPlatform.instance.setDefaultLanguage(lang);
  }

  Future<void> flush() {
    return AppticsFlutterPlatform.instance.flush();
  }
}