

import 'package:flutter/foundation.dart';

import '../apptics_flutter_platform_interface.dart';

class AppticsCrashTracker {


  static AppticsCrashTracker? _instance;

  static AppticsCrashTracker get instance {
    _instance ??= AppticsCrashTracker();
    return _instance!;
  }

  Future<void> autoCrashTracker() async {

    // Non-async exceptions
    FlutterError.onError = (details) {
      sendException(details.exception, details.stack,isFatal: true);
      // if (kReleaseMode) exit(1);
      FlutterError.presentError(details);
    };

    // Async exceptions
    PlatformDispatcher.instance.onError = (error, stack) {
      sendException(error, stack,isFatal: true);
      // if (kReleaseMode) exit(1);
      return true;
    };
  }

  Future<void> sendException(dynamic exception, StackTrace? stack,
      {dynamic reason,
        bool? isCrashPrint,
        bool isFatal = true}) async {

    // Checking print
    isCrashPrint ??= kDebugMode;


    if (isCrashPrint) {
      // Print Stack Trace
      print('Exception: $exception');

      if(stack != null) print("\n$stack");

    }

    final StackTrace stacktrace = (stack.toString().isEmpty || stack == null)
        ? StackTrace.current
        : stack;
    // To find build Architecture

    final pattern = RegExp(r'arch:\s*([\w\d]+)');
    final match = pattern.firstMatch(stack.toString());
    final arch = kReleaseMode ? match?.group(1) : null;

    return AppticsFlutterPlatform.instance.crashTracker(exception: exception.toString(), stacktrace: stacktrace.toString(), reason: reason.toString(), arch: arch, isFatal: isFatal);
  }

  Future<void> sendFlutterException(FlutterErrorDetails flutterErrorDetails,{bool isFatal = true}) async {
    FlutterError.presentError(flutterErrorDetails);
    sendException(flutterErrorDetails.exceptionAsString(), flutterErrorDetails.stack, isFatal: isFatal);
  }


  /// Record a non-fatal exception.
  /// Non-async exceptions should use this methods

  Future<void> sendFlutterNonFatalException(FlutterErrorDetails flutterErrorDetails) async {
    FlutterError.presentError(flutterErrorDetails);
    sendException(flutterErrorDetails.exceptionAsString(), flutterErrorDetails.stack, isFatal: false);
  }

  /// Record a non-fatal exception.
  /// Async exceptions should use this methods

  Future<void> sendNonFatalException(dynamic exception, StackTrace? stack) async {
    sendException(exception, stack, isFatal: false);
  }

  /// This method allows you to include custom information with your crash report that can help you troubleshoot the issue.

  /// @param properties any object that is conforms to JSON Encoding

  Future<void> setCrashCustomProperty(Map<String, dynamic> properties) {
    return AppticsFlutterPlatform.instance.setCrashCustomProperty(properties);
  }

  /// Call the below method on an appropriate screen in your app.
  /// This will present a pop-up only if the app has crashed in the previous sessions while the crash tracking was disabled.

  Future<void> showLastSessionCrashedPopup() {
    return AppticsFlutterPlatform.instance.showLastSessionCrashedPopup();
  }


  ///
  /// Provides the last app crash info.
  ///
  /// @return [String] Crash Info JSON as string.
  ///
  /// Sample crash info JSON
  ///   {
  ///   "issuename": "divide by zero",
  ///   "message": "java.lang.ArithmeticException: divide by zero\n\tat com.zoho.apptics.MainActivity.onCreate$lambda$2(MainActivity.kt:42)",
  ///   "networkstatus": 0,
  ///    "serviceprovider": "T-Mobile",
  ///   "orientation": 0,
  ///    "batterystatus": 100,
  ///    "edge": "Unknown",
  ///    "ram": "2.9 GB",
  ///    "rom": "5.8 GB",
  ///    "sessionstarttime": 1711445408267,
  ///    "customproperties": {},
  ///    "screenname": "com.zoho.apptics.MainActivity",
  ///    "happenedat": 1711445420908,
  ///    "happenedcount": 1,
  ///    "listofhappenedtime": 1711445420908,
  ///  "errortype": "native"
  ///    }
  ///
  Future<String?> getLastCrashInfo(){
    return AppticsFlutterPlatform.instance.getLastCrashInfo();
  }

  ///
  /// Try to sync crash instantly before app termination. Disabled by default.
  /// @param boolean.
  ///
  /// Sync will be attempted by blocking the crashing thread with a call timeout of 2 seconds.
  /// Turning On instant sync may produce ANRs and if sync not finished before app termination, it will
  /// be attempted again on next app launch or job scheduler resulting in duplicate crash reports.
  ///
  Future<void> setAttemptInstantSync(bool isEnable) {
    return AppticsFlutterPlatform.instance.setAttemptInstantSync(isEnable);
  }

}
